<?php

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/*
 * Plugin Name:       dev/null OpenStreetMap for Bricks
 * Plugin URI:        https://www.dev-null.at/bricks/open-street-maps
 * Description:       OpenStreetMap - Bricks element.
 * Version:           1.4.1
 * Requires at least: 6.7
 * Requires PHP:      8.0
 * Author:            Wolfgang Rotschek
 * Author URI:        https://www.dev-null.at/
 * License:           GPL v2 or later
 * License URI:       https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:       dev-null-openstreetmap
 * Domain Path:       /languages
 * Requires Plugins:  
 */

define( 'DEV_NULL_OPENSTREETMAP_JS_VERSION',    '1.3.1');
define( 'DEV_NULL_OPENSTREETMAP_PLUGIN_FILE',   __FILE__ );
define( 'DEV_NULL_OPENSTREETMAP_PLUGIN_DIR',    plugin_dir_path( __FILE__ ) );
define( 'DEV_NULL_OPENSTREETMAP_URL',           plugin_dir_url( __FILE__ ) );

define( 'DEV_NULL_LEAFLET_ICON',		'leaflet_icon');
define( 'DEV_NULL_LEAFLET_LOCATION', 	'leaflet_location');
define( 'DEV_NULL_LEAFLET_TYPE', 		'leaflet_location_type');
define( 'DEV_NULL_LEAFLET_GROUP', 		'leaflet_location_group');

load_plugin_textdomain( 'dev-null-openstreetmap', false, DEV_NULL_OPENSTREETMAP_PLUGIN_DIR . 'languages' );

require_once DEV_NULL_OPENSTREETMAP_PLUGIN_DIR . '/includes/init.php';

/**
 * Register custom elements
 */
add_action( 'init', function() {

	$dev_null_leaflet_includes = [
		'/shortcodes/dev_null_leaflet_map.php',
	];

	$dev_null_leaflet_elements = [
		__DIR__ . '/elements/element-leaflet-map.php',
	];

	foreach ( $dev_null_leaflet_includes as $file ) {
		require_once( __DIR__ . $file );
	}

	foreach ( $dev_null_leaflet_elements as $file ) {
		\Bricks\Elements::register_element( $file );
	}

    wp_register_script( 'dev_null_osm_admin_icon',
        DEV_NULL_OPENSTREETMAP_URL . '/assets/js/admin-icon.js',
        ['jquery'],
        DEV_NULL_OPENSTREETMAP_JS_VERSION,
        [
            'strategy' => 'defer',
            'in_footer' => true,
        ]);
        
	if(get_option('dev_null_osm_disable_js_leaflet') == 0) {
		wp_register_script( 'leaflet',
			DEV_NULL_OPENSTREETMAP_URL . 'assets/leaflet/leaflet.js',
			array(),
			'1.9.4',
            [
                'strategy' => 'defer',
                'in_footer' => true,
            ]);
		wp_register_style( 'leaflet',
			DEV_NULL_OPENSTREETMAP_URL . 'assets/leaflet/leaflet.css',
			array(),
			'1.9.4');
	} else {
		wp_register_script( 'leaflet', false);
		wp_register_style( 'leaflet', false);
	}
	
	if(get_option('dev_null_osm_disable_js_fullscreen') == 0) {
		wp_register_script( 'brunob-leaflet.fullscreen',
			DEV_NULL_OPENSTREETMAP_URL . 'assets/brunob.leaflet-fullscreen/Control.FullScreen.js',
			array('leaflet'),
			'4.0.0',
            [
                'strategy' => 'defer',
                'in_footer' => true,
            ]);

		wp_register_style( 'brunob-leaflet.fullscreen',
			DEV_NULL_OPENSTREETMAP_URL .  'assets/brunob.leaflet-fullscreen/Control.FullScreen.css',
			array('leaflet'),
			'4.0.0');
	} else {
		wp_register_script( 'brunob-leaflet.fullscreen', false);
		wp_register_style( 'brunob-leaflet.fullscreen', false);
	}
	
	if(get_option('dev_null_osm_disable_js_autocomplete') == 0) {
		wp_register_script( 'tomickigrzegorz.autocomplete',
			DEV_NULL_OPENSTREETMAP_URL . 'assets/tomickigrzegorz.autocomplete/dist/js/autocomplete.js',
			array('leaflet'),
			'2.0.3',
            [
                'strategy' => 'defer',
                'in_footer' => true,
            ]);
		wp_register_style( 'tomickigrzegorz.autocomplete',
			DEV_NULL_OPENSTREETMAP_URL . 'assets/tomickigrzegorz.autocomplete/dist/css/autocomplete.css',
			array('leaflet'),
			'2.0.3');
	} else {
		wp_register_script( 'tomickigrzegorz.autocomplete', false);
		wp_register_style( 'tomickigrzegorz.autocomplete', false);
	}
	
	if(get_option('dev_null_osm_disable_js_cluster') == 0) {
		wp_register_script( 'leaflet.markercluster',
			DEV_NULL_OPENSTREETMAP_URL . 'assets/leaflet.markercluster/leaflet.markercluster.js',
			array('leaflet'),
			'1.4.1');
		wp_register_style( 'MarkerCluster.Spider',
			DEV_NULL_OPENSTREETMAP_URL . 'assets/leaflet.markercluster/MarkerCluster.css',
			array('leaflet'),
			'1.4.1');
		wp_register_style( 'MarkerCluster.Default',
			DEV_NULL_OPENSTREETMAP_URL . 'assets/leaflet.markercluster/MarkerCluster.Default.css',
			array('leaflet'),
			'1.4.1');
	} else {
		wp_register_script( 'leaflet.markercluster', false);
		wp_register_style( 'MarkerCluster.Spider', false);
		wp_register_style( 'MarkerCluster.Default', false);
	}
					  
	if(get_option('dev_null_osm_disable_js_kml') == 0) {
		wp_register_script('dev-null-osm-kml',
			DEV_NULL_OPENSTREETMAP_URL . 'assets/windycom.leaflet-kml/L.KML.js',
			array('leaflet'),
			'1.1.0');
	} else {
		wp_register_script( 'dev-null-osm-kml', false);
	}
	
	if(get_option('dev_null_osm_disable_js_gpx') == 0) {
	    wp_register_script('dev-null-osm-gpx',
			DEV_NULL_OPENSTREETMAP_URL . 'assets/mpetazzoni.leaflet-gpx/gpx.js',
			array('leaflet'),
			'2.1.2');
	}
	else
	{
	    wp_register_script('dev-null-osm-gpx', false);
	}
	
	wp_register_script( 'dev-null-openstreetmap', false, 
		[
			'leaflet',
			'brunob-leaflet.fullscreen',
			'tomickigrzegorz.autocomplete',
			'dev-null-osm-kml',
			'dev-null-osm-gpx',
			'leaflet.markercluster',
		]);
	wp_register_style( 'dev-null-openstreetmap', false,
		[
			'leaflet',
		 	'brunob-leaflet.fullscreen',
			'MarkerCluster.Spider',
			'MarkerCluster.Default',
		]);
    
}, 11 );

function dev_null_openstreetmap_plugin_activate() {

	$iconsBase = DEV_NULL_OPENSTREETMAP_PLUGIN_DIR . 'assets/leaflet-markers';

	$query = array(
		'post_type'        	=> array( DEV_NULL_LEAFLET_ICON ),
		'posts_per_page'	=> -1,
	);
			
	$icons = get_posts($query);

	$list = [];
	
	foreach($icons as $icon) {
		$list[$icon->post_name] = array(
			'ID'	=> $icon->ID,
			'title' => $icon->post_title,
		);
	}

	$folders = [
		[
			'folder' 			=> '/needles/',
			'icon_size_x'	    => 48,
			'icon_size_y'	    => 48,
			'icon_anchor_x'		=> 24,
			'icon_anchor_y'		=> 46,
			'popup_anchor_x'	=> 1,
			'popup_anchor_y'	=> -34,
		],[
			'folder' => '/pins/',
			'icon_size_x'	    => 48,
			'icon_size_y'	    => 48,
			'icon_anchor_x'		=> 24,
			'icon_anchor_y'		=> 46,
			'popup_anchor_x'	=> 1,
			'popup_anchor_y'	=> -34,
		],[
			'folder' => '/mapsmarker.com/',
			'icon_size_x'	    => 32,
			'icon_size_y'	    => 37,
			'icon_anchor_x'		=> 16,
			'icon_anchor_y'		=> 37,
			'popup_anchor_x'	=> 1,
			'popup_anchor_y'	=> -34,
		],[
			'folder' => '/nchart/',
			'icon_size_x'	    => 41,
			'icon_size_y'	    => 41,
			'icon_anchor_x'		=> 20,
			'icon_anchor_y'		=> 20,
			'popup_anchor_x'	=> 0,
			'popup_anchor_y'	=> -10,
		],
	];
	
	foreach($folders as $folder)
	{
		$files = scandir($iconsBase . $folder['folder']);

		foreach($files as $file) {
			$base = pathinfo($file);
			
			if($base['extension'] != 'png')
				continue;
		
			$base['title'] = ucfirst(trim(str_replace(['-', '_'], ' ', $base['filename'])));
			$postArray = [
				'post_title'   => $base['title'],
				'post_content' => '',
				'post_status'  => 'publish',
				'post_author'  => get_current_user_id(),
				'meta_input'   => [
					'icon_url' 			=> plugins_url('/assets/leaflet-markers', __FILE__ ) . $folder['folder'] . $file,
					'icon_size_x'	    => $folder['icon_size_x'],
					'icon_size_y'	    => $folder['icon_size_y'],
					'icon_anchor_x'		=> $folder['icon_anchor_x'],
					'icon_anchor_y'		=> $folder['icon_anchor_y'],
					'popup_anchor_x'	=> $folder['popup_anchor_x'],
					'popup_anchor_y'	=> $folder['popup_anchor_y'],

				],
				'post_type'		=> DEV_NULL_LEAFLET_ICON,
			];

			$slug = str_replace([' ', '_', '.'], '-', $base['filename']);
			
			if(isset($list[$slug])) {
				$postArray['ID']	= $list[$slug]['ID'];
			} else {
				$postArray['ID']	= 0;
			}
			wp_insert_post($postArray);
		}
	}
	return;
  
}

register_activation_hook( __FILE__, 'dev_null_openstreetmap_plugin_activate' );