<?php

namespace DevNullOpenStreetMap;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class Plugin {

	public $location_type;
	public $location_group;
	public $leaflet_icon;
	public $leaflet_location;
	public $leaflet_layer_kml;
	public $leaflet_layer_gpx;
	public $leaflet_country;
    
	/**
	 * The one and only Theme instance
	 *
	 * @var Theme
	 */
	public static $instance;

	/**
	 * Autoload and init components
	 */
	public function __construct() {
		$this->autoloader();
		$this->init();
	}

	/**
	 * Autoload files
	 */
	private function autoloader() {
		require_once DEV_NULL_OPENSTREETMAP_PLUGIN_DIR . 'includes/autoloader.php';

		Autoloader::register();
	}

	/**
	 * Init components
	 */
	public function init() {
		$leaflet_icon 		= new \DevNullOpenStreetMap\Leaflet_Icon();
		$location_group 	= new \DevNullOpenStreetMap\Leaflet_Location_Group();
		$location_type 		= new \DevNullOpenStreetMap\Leaflet_Location_Type();
		$leaflet_location 	= new \DevNullOpenStreetMap\Leaflet_Location_Item();
        $leaflet_layer_kml 	= new \DevNullOpenStreetMap\Leaflet_Layer_Kml();
        $leaflet_layer_gpx 	= new \DevNullOpenStreetMap\Leaflet_Layer_Gpx();
		$leaflet_country	= new \DevNullOpenStreetMap\Leaflet_Location_Country();
	}
	
	/**
	 * Main Plugin instance
	 *
	 * Ensure only one instance of Plugin exists at any given time.
	 *
	 * @return object Plugin The one and only Plugin instance
	 */
	public static function instance() {

		if ( ! isset( self::$instance ) && ! ( self::$instance instanceof Plugin ) ) {
			self::$instance = new self();
		}

		return self::$instance;

	}
}

// Get the plugin up and running
Plugin::instance();

