<?php

namespace DevNullOpenStreetMap;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class Leaflet_Icon {

	public static $markerIcons           = false;

	public function __construct() {
		add_action( 'init', [ $this, 'register_post_type' ] );

	}
	
	public static function get_IconNamesList() {

		$query = array(
			'post_type'        	=> array( DEV_NULL_LEAFLET_ICON ),
			'posts_per_page'	=> -1,
			'orderby'			=> 'title',
		);
		$icons = get_posts($query);

		foreach($icons as $icon) {
			$list[$icon->post_name] = $icon->post_title;
		}

		return $list;
	}
	
	public static function get_IconScript($name='.leaflet') {
					
		if(self::$markerIcons == false) {
			$query = array(
				'post_type'        	=> array( DEV_NULL_LEAFLET_ICON ),
				'posts_per_page'	=> -1,
			);
			
			$icons = get_posts($query);

			foreach($icons as $icon) {
				$list[$icon->post_name] = array(
											'ID'	=> $icon->ID,
											'meta'	=> array(),
											'title' => esc_attr($icon->post_title),
											'icon_url' => false,
											);
			}
			
			self::$markerIcons = $list;
		}

		if(isset(self::$markerIcons[$name]) == false) {

			self::$markerIcons[$name] = array(
											'ID'	=> false,
											'title' => esc_attr($name),
											'meta'	=> array(),
											'icon_url' => false,
											);
										
		}

		if(array_key_exists($name, self::$markerIcons)) {
			$id 	= self::$markerIcons[$name]['ID'];
			
			$icon	= &self::$markerIcons[$name];
			
			if(self::$markerIcons[$name]['icon_url'] == false) {
				$icon['icon_url']		    = $id ? get_post_meta($id, 'icon_url', true) :
					DEV_NULL_OPENSTREETMAP_URL . 'assets/leaflet/images/marker-icon.png';
				$icon['icon_size_x']	    = $id ? get_post_meta($id, 'icon_size_x', true) 	: 25;
				$icon['icon_size_y']	    = $id ? get_post_meta($id, 'icon_size_y', true) 	: 41;
				$icon['icon_anchor_x']	    = $id ? get_post_meta($id, 'icon_anchor_x', true) 	: 12;
				$icon['icon_anchor_y']	    = $id ? get_post_meta($id, 'icon_anchor_y', true) 	: 41;
				$icon['popup_anchor_x']	    = $id ? get_post_meta($id, 'popup_anchor_x', true) 	: 1;
				$icon['popup_anchor_y']	    = $id ? get_post_meta($id, 'popup_anchor_y', true) 	: -34;
				$icon['shadow_url']		    = $id ? get_post_meta($id, 'shadow_url', true) :
					DEV_NULL_OPENSTREETMAP_URL . 'assets/leaflet/images/marker-shadow.png';
				$icon['shadow_size_x']	    = $id ? get_post_meta($id, 'shadow_size_x', true) 	: 41;
				$icon['shadow_size_y']	    = $id ? get_post_meta($id, 'shadow_size_y', true) 	: 41;
				$icon['shadow_anchor_x']	= $id ? get_post_meta($id, 'shadow_anchor_x', true)	: 0;
				$icon['shadow_anchor_y']	= $id ? get_post_meta($id, 'shadow_anchor_y', true)	: 41;
			}
		}
		
		$script = '';
		
		$iconUrl		= esc_attr(self::$markerIcons[$name]['icon_url']);
		$iconSizeX		= esc_attr(self::$markerIcons[$name]['icon_size_x']);
		$iconSizeY 		= esc_attr(self::$markerIcons[$name]['icon_size_y']);
		$iconAnchorX 	= esc_attr(self::$markerIcons[$name]['icon_anchor_x']);
		$iconAnchorY 	= esc_attr(self::$markerIcons[$name]['icon_anchor_y']);
		$PopupAnchorX 	= esc_attr(self::$markerIcons[$name]['popup_anchor_x']);
		$PopupAnchorY 	= esc_attr(self::$markerIcons[$name]['popup_anchor_y']);
		$shadowUrl		= esc_attr(self::$markerIcons[$name]['shadow_url']);
		$shadowSizeX	= esc_attr(self::$markerIcons[$name]['shadow_size_x']);
		$shadowSizeY 	= esc_attr(self::$markerIcons[$name]['shadow_size_y']);
		$shadowAnchorX	= esc_attr(self::$markerIcons[$name]['shadow_anchor_x']);
		$shadowAnchorY 	= esc_attr(self::$markerIcons[$name]['shadow_anchor_y']);

		$script = "
icons['{$name}'] = new L.Icon({
	iconUrl: '$iconUrl',
	iconSize: [{$iconSizeX}, {$iconSizeY}],
	iconAnchor: [{$iconAnchorX}, {$iconAnchorY}],
	popupAnchor: [{$PopupAnchorX}, {$PopupAnchorY}],

	";

		if($shadowUrl <> '') {
			$script .= "
	shadowUrl: '{$shadowUrl}',
	shadowSize: [{$shadowSizeX}, {$shadowSizeX}],
    shadowAnchor: [$shadowAnchorX, $shadowAnchorY]
	";

		}

		$script .= "
});
";

		return $script;
	}

	public static function manage_columns( $columns ) {
		$columns = [
			'cb'           		=> '<input type="checkbox" />',
			'title'        		=> esc_html__( 'Title', 'dev-null-openstreetmap' ),
			'icon' 				=> esc_html__( 'Icon', 'dev-null-openstreetmap' ),
			'icon_size' 		=> esc_html__( 'Icon Size', 'dev-null-openstreetmap' ),
			'icon_anchor' 		=> esc_html__( 'Icon Anchor', 'dev-null-openstreetmap' ),
			'popup_anchor' 		=> esc_html__( 'Popup Anchor', 'dev-null-openstreetmap' ),
			'shadow_size' 		=> esc_html__( 'Shadow Size', 'dev-null-openstreetmap' ),
			'shadow_anchor'     => esc_html__( 'Shadow Anchor', 'dev-null-openstreetmap' ),
		];

		return $columns;
	}

	public static function render_columns( $column, $post_id ) {

		switch($column) {
		case 'icon':
			$icon = get_post_meta($post_id, 'icon_url', true);
			if(empty($icon))
				$icon = DEV_NULL_OPENSTREETMAP_URL . 'assets/leaflet/images/marker-icon.png';
			
			$icon = esc_url($icon);
			echo "<img src='" . esc_url($icon) . "' style='width:36px'>";
			break;
		case 'icon_anchor':
			echo esc_html( '[ '
				. get_post_meta($post_id, 'icon_anchor_x', true) 
				. ' , '
				. get_post_meta($post_id, 'icon_anchor_y', true) 
				. ' ]');
			break;
		case 'icon_size':
			echo esc_html( '[ '
				. get_post_meta($post_id, 'icon_size_x', true) 
				. ' , '
				. get_post_meta($post_id, 'icon_size_y', true) 
				. ' ]');
			break;
		case 'popup_anchor':
			echo esc_html( '[ '
				. get_post_meta($post_id, 'popup_anchor_x', true) 
				. ' , '
				. get_post_meta($post_id, 'popup_anchor_y', true) 
				. ' ]');
			break;
		case 'shadow_size':
			echo esc_html( '[ '
				. get_post_meta($post_id, 'shadow_size_x', true) 
				. ' , '
				. get_post_meta($post_id, 'shadow_size_y', true) 
				. ' ]');
			break;
		case 'shadow_anchor':
			echo esc_html( '[ '
				. get_post_meta($post_id, 'shadow_anchor_x', true) 
				. ' , '
				. get_post_meta($post_id, 'shadow_anchor_y', true) 
				. ' ]');
			break;
		}

	}

	public static function post_row_actions( $actions, $post ) {

		if($post->post_type === DEV_NULL_LEAFLET_ICON) {			
			// Remove 'Quick Edit'
			unset( $actions['inline hide-if-no-js'] );
			// Remove 'View'
			unset( $actions['view'] );
		}
		
		return $actions;
	}

	
	function register_post_type() {
		$labels = [
			'name'                     => esc_html__( 'Leaflet Icons', 'dev-null-openstreetmap' ),
			'singular_name'            => esc_html__( 'Leaflet Icon', 'dev-null-openstreetmap' ),
			'add_new'                  => esc_html__( 'Add New', 'dev-null-openstreetmap' ),
			'add_new_item'             => esc_html__( 'Add New Leaflet Icon', 'dev-null-openstreetmap' ),
			'edit_item'                => esc_html__( 'Edit Leaflet Icon', 'dev-null-openstreetmap' ),
			'new_item'                 => esc_html__( 'New Leaflet Icon', 'dev-null-openstreetmap' ),
			'view_item'                => esc_html__( 'View Leaflet Icon', 'dev-null-openstreetmap' ),
			'view_items'               => esc_html__( 'View Leaflet Icons', 'dev-null-openstreetmap' ),
			'search_items'             => esc_html__( 'Search Leaflet Icons', 'dev-null-openstreetmap' ),
			'not_found'                => esc_html__( 'No leaflet icons found.', 'dev-null-openstreetmap' ),
			'not_found_in_trash'       => esc_html__( 'No leaflet icons found in Trash.', 'dev-null-openstreetmap' ),
			'all_items'                => esc_html__( 'Leaflet Icons', 'dev-null-openstreetmap' ),
			'menu_name'                => esc_html__( 'Leaflet Icons', 'dev-null-openstreetmap' ),
			'filter_items_list'        => esc_html__( 'Filter leaflet icons list', 'dev-null-openstreetmap' ),
			'items_list'               => esc_html__( 'Leaflet Icons list', 'dev-null-openstreetmap' ),
			'item_published'           => esc_html__( 'Leaflet Icon published.', 'dev-null-openstreetmap' ),
			'item_published_privately' => esc_html__( 'Leaflet Icon published privately.', 'dev-null-openstreetmap' ),
			'item_reverted_to_draft'   => esc_html__( 'Leaflet Icon reverted to draft.', 'dev-null-openstreetmap' ),
			'item_scheduled'           => esc_html__( 'Leaflet Icon scheduled.', 'dev-null-openstreetmap' ),
			'item_updated'             => esc_html__( 'Leaflet Icon updated.', 'dev-null-openstreetmap' ),
			'text_domain'              => esc_html__( 'dev-null-openstreetmap', 'dev-null-openstreetmap' ),
		];
		$args = [
			'label'               => esc_html__( 'Leaflet Icons', 'dev-null-openstreetmap' ),
			'labels'              => $labels,
			'description'         => '',
			'public'              => true,
			'publicly_queryable'  => true,
			'show_ui'             => true,
			'show_in_menu'        => 'edit.php?post_type=' . DEV_NULL_LEAFLET_LOCATION,
			'show_in_nav_menus'   => false,
			'show_in_admin_bar'   => true,
			'exclude_from_search' => true,
			'hierarchical'        => false,
			'has_archive'		  => false,
			'query_var'           => true,
			'rewrite'             => ['with_front' => false],
			'menu_icon'           => 'dashicons-location',
			'capability_type'     => 'post',
			'supports'            => [ 'title' ],
		];

		register_post_type( DEV_NULL_LEAFLET_ICON, $args );
	}
    
	/**
	 * Set up and add the meta box.
	 */
	public static function add_meta_boxes() {
        
        add_meta_box(
            DEV_NULL_LEAFLET_ICON,   // Unique ID
            esc_html__( 'Leaflet Icon', 'dev-null-openstreetmap' ),    // Box title
            [ self::class, 'html' ],    // Content callback, must be of type callable
            DEV_NULL_LEAFLET_ICON    // Post type
        );
	}


	/**
	 * Save the meta box selections.
	 *
	 * @param int $post_id  The post ID.
	 */
	public static function save( int $post_id ) {
        if ( ! isset( $_POST['leaflet-icon-nonce'] ) || ! wp_verify_nonce( sanitize_text_field( wp_unslash ( $_POST['leaflet-icon-nonce'] ) ) , basename( __FILE__ ) ) )
        {
            return;
        }

		$icon_url			= isset($_POST['icon_url'])			? sanitize_url(wp_unslash($_POST['icon_url']))	: '';
		$icon_size_x 		= isset($_POST['icon_size_x']) 		? intval($_POST['icon_size_x']) 	: '';
		$icon_size_y 		= isset($_POST['icon_size_y']) 		? intval($_POST['icon_size_y']) 	: '';
		$icon_anchor_x 		= isset($_POST['icon_anchor_x']) 	? intval($_POST['icon_anchor_x']) 	: '';
		$icon_anchor_y 		= isset($_POST['icon_anchor_y']) 	? intval($_POST['icon_anchor_y']) 	: '';
		$popup_anchor_x 	= isset($_POST['popup_anchor_x']) 	? intval($_POST['popup_anchor_x']) 	: '';
		$popup_anchor_y 	= isset($_POST['popup_anchor_y']) 	? intval($_POST['popup_anchor_y']) 	: '';
		
		$shadow_url			= isset($_POST['shadow_url'])		? sanitize_url(wp_unslash($_POST['shadow_url'])) : '';
		$shadow_size_x 		= isset($_POST['shadow_size_x']) 	? intval($_POST['shadow_size_x']) 	: '';
		$shadow_size_y 		= isset($_POST['shadow_size_y']) 	? intval($_POST['shadow_size_y']) 	: '';
		$shadow_anchor_x	= isset($_POST['shadow_anchor_x']) 	? intval($_POST['shadow_anchor_x']) : '';
		$shadow_anchor_y	= isset($_POST['shadow_anchor_y']) 	? intval($_POST['shadow_anchor_y']) : '';
        
        update_post_meta($post_id, 'icon_url',      	$icon_url);
        update_post_meta($post_id, 'icon_size_x',   	$icon_size_x);
        update_post_meta($post_id, 'icon_size_y',   	$icon_size_y);
        update_post_meta($post_id, 'icon_anchor_x', 	$icon_anchor_x);
        update_post_meta($post_id, 'icon_anchor_y', 	$icon_anchor_y);
        update_post_meta($post_id, 'popup_anchor_x', 	$popup_anchor_x);
        update_post_meta($post_id, 'popup_anchor_y', 	$popup_anchor_y);
        
		update_post_meta($post_id, 'shadow_url',    	$shadow_url);
        update_post_meta($post_id, 'shadow_size_x', 	$shadow_size_x);
        update_post_meta($post_id, 'shadow_size_y', 	$shadow_size_y);
        update_post_meta($post_id, 'shadow_anchor_x', 	$shadow_anchor_x);
        update_post_meta($post_id, 'shadow_anchor_y', 	$shadow_anchor_y);	

	}


	/**
	 * Display the meta box HTML to the user.
	 *
	 * @param WP_Post $post   Post object.
	 */
	public static function html( $post ) {

        wp_nonce_field( basename( __FILE__ ), 'leaflet-icon-nonce' );
        		
?>

<div class="dev-null-meta-box" data-autosave="false" data-object-type="post">
    <div class="dev-null-field dev-null-file_input-wrapper">
        <div class="dev-null-label" id="icon_url-label"><label for="icon_url">Icon URL</label></div>
        <div class="dev-null-input" data-clone-empty-start="0">
            <div class="rwmb-file-input-image ">
                <img src="<?php echo esc_url(get_post_meta( $post->ID, 'icon_url', true)); ?>">
            </div>
            <div class="rwmb-file-input-inner">
                <input value="<?php echo esc_url(get_post_meta( $post->ID, 'icon_url', true)); ?>" type="text" id="icon_url" class="rwmb-file_input" name="icon_url" aria-labelledby="icon_url-label">
                <a id="upload_image_button" href="#" class="rwmb-file-input-select button">Auswählen</a>
                <a href="#" class="rwmb-file-input-remove button ">Entfernen</a>
            </div>
        </div>
    </div>
    <div class="dev-null-field dev-null-text-wrapper">
        <div class="dev-null-label" id="icon_size_x-label"><label for="icon_size_x">Icon Size X</label></div>
        <div class="dev-null-input" data-clone-empty-start="0"><input value="<?php echo esc_attr(get_post_meta( $post->ID, 'icon_size_x', true)); ?>" type="text" id="icon_size_x" class="dev-null-text" name="icon_size_x" aria-labelledby="icon_size_x-label"></div>
    </div>
    <div class="dev-null-field dev-null-text-wrapper">
        <div class="dev-null-label" id="icon_size_y-label"><label for="icon_size_y">Icon Size Y</label></div>
        <div class="dev-null-input" data-clone-empty-start="0"><input value="<?php echo esc_attr(get_post_meta( $post->ID, 'icon_size_y', true)); ?>" type="text" id="icon_size_y" class="dev-null-text" name="icon_size_y" aria-labelledby="icon_size_y-label"></div>
    </div>
    <div class="dev-null-field dev-null-text-wrapper">
        <div class="dev-null-label" id="icon_anchor_x-label"><label for="icon_anchor_x">Icon Anchor X</label></div>
        <div class="dev-null-input" data-clone-empty-start="0"><input value="<?php echo esc_attr(get_post_meta( $post->ID, 'icon_anchor_x', true)); ?>" type="text" id="icon_anchor_x" class="dev-null-text" name="icon_anchor_x" aria-labelledby="icon_anchor_x-label"></div>
    </div>
    <div class="dev-null-field dev-null-text-wrapper">
        <div class="dev-null-label" id="icon_anchor_y-label"><label for="icon_anchor_y">Icon Anchor Y</label></div>
        <div class="dev-null-input" data-clone-empty-start="0"><input value="<?php echo esc_attr(get_post_meta( $post->ID, 'icon_anchor_y', true)); ?>" type="text" id="icon_anchor_y" class="dev-null-text" name="icon_anchor_y" aria-labelledby="icon_anchor_y-label"></div>
    </div>
    <div class="dev-null-field dev-null-text-wrapper">
        <div class="dev-null-label" id="popup_anchor_x-label"><label for="popup_anchor_x">Popup Anchor X</label></div>
        <div class="dev-null-input" data-clone-empty-start="0"><input value="<?php echo esc_attr(get_post_meta( $post->ID, 'popup_anchor_x', true)); ?>" type="text" id="popup_anchor_x" class="dev-null-text" name="popup_anchor_x" aria-labelledby="popup_anchor_x-label"></div>
    </div>
    <div class="dev-null-field dev-null-text-wrapper">
        <div class="dev-null-label" id="popup_anchor_y-label"><label for="popup_anchor_y">Popup Anchor Y</label></div>
        <div class="dev-null-input" data-clone-empty-start="0"><input value="<?php echo esc_attr(get_post_meta( $post->ID, 'popup_anchor_y', true)); ?>" type="text" id="popup_anchor_y" class="dev-null-text" name="popup_anchor_y" aria-labelledby="popup_anchor_y-label"></div>
    </div>
    <div class="dev-null-field dev-null-file_input-wrapper">
        <div class="dev-null-label" id="shadow_url-label"><label for="shadow_url">Shadow URL</label></div>
        <div class="dev-null-input" data-clone-empty-start="0">
            <div class="rwmb-file-input-image">
                <img src="<?php echo esc_url(get_post_meta( $post->ID, 'shadow_url', true)); ?>">
            </div>
            <div class="rwmb-file-input-inner">
                <input value="<?php echo esc_url(get_post_meta( $post->ID, 'shadow_url', true)); ?>" type="text" id="shadow_url" class="rwmb-file_input" name="shadow_url" aria-labelledby="shadow_url-label">
                <a id="upload_shadow_button" href="#" class="rwmb-file-input-select button">Auswählen</a>
                <a href="#" class="rwmb-file-input-remove button ">Entfernen</a>
            </div>
        </div>
    </div>
    <div class="dev-null-field dev-null-text-wrapper">
        <div class="dev-null-label" id="shadow_size_x-label"><label for="shadow_size_x">Shadow Size X</label></div>
        <div class="dev-null-input" data-clone-empty-start="0"><input value="<?php echo esc_attr(get_post_meta( $post->ID, 'shadow_size_x', true)); ?>" type="text" id="shadow_size_x" class="dev-null-text" name="shadow_size_x" aria-labelledby="shadow_size_x-label"></div>
    </div>
    <div class="dev-null-field dev-null-text-wrapper">
        <div class="dev-null-label" id="shadow_size_y-label"><label for="shadow_size_y">Shadow Size Y</label></div>
        <div class="dev-null-input" data-clone-empty-start="0"><input value="<?php echo esc_attr(get_post_meta( $post->ID, 'shadow_size_y', true)); ?>" type="text" id="shadow_size_y" class="dev-null-text" name="shadow_size_y" aria-labelledby="shadow_size_y-label"></div>
    </div>
    <div class="dev-null-field dev-null-text-wrapper">
        <div class="dev-null-label" id="shadow_anchor_x-label"><label for="shadow_anchor_x">Shadow Anchor X</label></div>
        <div class="dev-null-input" data-clone-empty-start="0"><input value="<?php echo esc_attr(get_post_meta( $post->ID, 'shadow_anchor_x', true)); ?>" type="text" id="shadow_anchor_x" class="dev-null-text" name="shadow_anchor_x" aria-labelledby="shadow_anchor_x-label"></div>
    </div>
    <div class="dev-null-field dev-null-text-wrapper">
        <div class="dev-null-label" id="shadow_anchor_y-label"><label for="shadow_anchor_y">Shadow Anchor Y</label></div>
        <div class="dev-null-input" data-clone-empty-start="0"><input value="<?php echo esc_attr(get_post_meta( $post->ID, 'shadow_anchor_y', true)); ?>" type="text" id="shadow_anchor_y" class="dev-null-text" name="shadow_anchor_y" aria-labelledby="shadow_anchor_y-label"></div>
    </div>
</div>

<?php

	}
    
    static function admin_enqueue_scripts($hook) {

       if(get_post_type() != DEV_NULL_LEAFLET_ICON)
            return;
        
        wp_enqueue_media();
        wp_enqueue_style('leaflet-icon', DEV_NULL_OPENSTREETMAP_URL . '/assets/css/leaflet-markers.css');
        
        wp_enqueue_script('dev_null_osm_admin_icon');
        
        wp_enqueue_script('dummy', get_post_type());
    }
}

add_action( 'admin_enqueue_scripts',
    [ '\DevNullOpenStreetMap\Leaflet_Icon', 'admin_enqueue_scripts' ],
    10, 1 );

add_filter( 'post_row_actions', 
    [ '\DevNullOpenStreetMap\Leaflet_Icon', 'post_row_actions' ],
    10, 2 );

add_filter( 'manage_' . DEV_NULL_LEAFLET_ICON . '_posts_columns',
    [ '\DevNullOpenStreetMap\Leaflet_Icon', 'manage_columns' ],
    10, 1 );
    
add_action( 'manage_' . DEV_NULL_LEAFLET_ICON . '_posts_custom_column',
    [ '\DevNullOpenStreetMap\Leaflet_Icon', 'render_columns' ],
    10, 2 );

add_action( 'add_meta_boxes_' . DEV_NULL_LEAFLET_ICON,
    [ '\DevNullOpenStreetMap\Leaflet_Icon', 'add_meta_boxes' ],
    10, 2 );
    
add_action( 'save_post_' . DEV_NULL_LEAFLET_ICON,
    [ '\DevNullOpenStreetMap\Leaflet_Icon', 'save' ],
    10, 3 );
