<?php

namespace DevNullOpenStreetMap;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class Leaflet_Location_Item {

	public function __construct() {

		add_action( 'init', [ $this, 'register_post_type'] );

	}

	public static function load_items($items, $list)
	{
		if(is_array($list) == false)
			$list = explode(',', $list);

		$list = array_map('intval', $list);
		
		if(count($list) == 0)
			return $items;
		
		$query = [
			'post__in'	=> $list,
			'post_type'	=> DEV_NULL_LEAFLET_LOCATION,
			'nopaging'			=> true,
		];
		
		$posts = get_posts($query);
		
		foreach($posts as $p) {
			$items[$p->ID] = $p;
		}
		
		return $items;
	}
	
	public static function get_LocationItemList() {

		$query = array(
			'post_type'        	=> array( DEV_NULL_LEAFLET_LOCATION ),
			'post_status'       => array( 'publish' ),
			'posts_per_page'	=> -1,
		);
		$locations = $locations = get_posts($query);	

		$list = array();
		
		foreach($locations as $loc) {
			$list[$loc->ID] = $loc->post_title;
		}
		return $list;
	}
	
    public static function item_list_points($items) {
        
		$query = array(
			'post_type'        	=> array( DEV_NULL_LEAFLET_LOCATION ),
			'post_status'       => array( 'publish' ),
			'posts_per_page'	=> -1,
		);
		$locations = $locations = get_posts($query);	

		$list = array();
		
		foreach($locations as $loc) {
			$key = 'pnt:' . $loc->ID;
			
			if(array_key_exists($key, $items))
				continue;
			
			$ctr = get_post_meta($loc->ID, 'location_countrycode', true);
			if($ctr != false && $ctr != '') {
				$ctr = strtoupper($ctr);
				$items['ctr:' . $ctr] = __ ('Countrycode', 'dev-null-openstreetmap') . ' ' . esc_attr($ctr) . ' [ctr]';
			}
			
			if($loc->post_title != '')
				$items['pnt:' . $loc->ID] = esc_attr($loc->post_title) . ' [pnt]';
		}
		return $items;
        
    }
    
	public static function post_row_actions( $actions, $post ) {

		// Remove 'Quick Edit'
		if ( $post->post_type === DEV_NULL_LEAFLET_LOCATION ) {
			unset( $actions['inline hide-if-no-js'] );
//			unset( $actions['view'] );
		}

		return $actions;
	}
	
	function register_post_type() {
		$labels = [
			'name'                     => esc_html__( 'Locations', 'dev-null-openstreetmap' ),
			'singular_name'            => esc_html__( 'Location', 'dev-null-openstreetmap' ),
			'add_new'                  => esc_html__( 'Add New', 'dev-null-openstreetmap' ),
			'add_new_item'             => esc_html__( 'Add New Location', 'dev-null-openstreetmap' ),
			'edit_item'                => esc_html__( 'Edit Location', 'dev-null-openstreetmap' ),
			'new_item'                 => esc_html__( 'New Location', 'dev-null-openstreetmap' ),
			'view_item'                => esc_html__( 'View Location', 'dev-null-openstreetmap' ),
			'view_items'               => esc_html__( 'View Locations', 'dev-null-openstreetmap' ),
			'search_items'             => esc_html__( 'Search Locations', 'dev-null-openstreetmap' ),
			'not_found'                => esc_html__( 'No locations found.', 'dev-null-openstreetmap' ),
			'not_found_in_trash'       => esc_html__( 'No locations found in Trash.', 'dev-null-openstreetmap' ),
			'all_items'                => esc_html__( 'All Locations', 'dev-null-openstreetmap' ),
			'menu_name'                => esc_html__( 'Locations', 'dev-null-openstreetmap' ),
			'text_domain'              => esc_html__( 'dev-null-openstreetmap', 'dev-null-openstreetmap' ),
		];
		$args = [
			'label'               => esc_html__( 'Locations', 'dev-null-openstreetmap' ),
			'labels'              => $labels,
			'description'         => '',
			'public'              => true,
			'publicly_queryable'  => true,
			'hierarchical'        => false,
			'exclude_from_search' => true,
			'show_ui'             => true,
			'show_in_nav_menus'   => false,
			'show_in_admin_bar'   => true,
//			'show_in_rest'        => true,
			'query_var'           => true,
			'can_export'          => true,
			'delete_with_user'    => false,
			'has_archive'         => false,
			'rest_base'           => '',
			'show_in_menu'        => true,
			'menu_position'       => '',
			'menu_icon'           => 'dashicons-location-alt',
			'capability_type'     => 'post',
			'supports'            => ['title'],
			'taxonomies'          => [DEV_NULL_LEAFLET_TYPE, DEV_NULL_LEAFLET_GROUP],
			'rewrite'             => [
				'with_front' => false,
			],
		];

		register_post_type( DEV_NULL_LEAFLET_LOCATION, $args );
	}
    
	/**
	 * Set up and add the meta box.
	 */
	public static function add_meta_boxes() {
        add_meta_box(
            DEV_NULL_LEAFLET_LOCATION,   // Unique ID
            esc_html__( 'Location', 'dev-null-openstreetmap' ),    // Box title
            [ self::class, 'html' ],    // Content callback, must be of type callable
            DEV_NULL_LEAFLET_LOCATION    // Post type
        );
	}
    
	/**
	 * Save the meta box selections.
	 *
	 * @param int $post_id  The post ID.
	 */
	public static function save( int $post_id ) {
		
        if ( ! isset( $_POST['leaflet-location-nonce'] ) || ! wp_verify_nonce( sanitize_text_field( wp_unslash ( $_POST['leaflet-location-nonce'] ) ) , basename( __FILE__ ) ) )
        {
            return;
        }
  
		$address_osm			= isset($_POST['address_osm'])			? sanitize_text_field(wp_unslash($_POST['address_osm'])) 		: '';
		$location_longitude		= isset($_POST['location_longitude'])	? floatval($_POST['location_longitude'])			: 0.0;
		$location_latitude		= isset($_POST['location_latitude'])	? floatval($_POST['location_latitude'])				: 0.0;
		$location_address		= isset($_POST['location_address'])		? sanitize_text_field(wp_unslash($_POST['location_address']))	: '';
		$location_city			= isset($_POST['location_city'])		? sanitize_text_field(wp_unslash($_POST['location_city']))		: '';
		$location_postcode		= isset($_POST['location_postcode'])	? sanitize_text_field(wp_unslash($_POST['location_postcode']))	: '';
		$location_countrycode	= isset($_POST['location_countrycode'])	? sanitize_text_field(wp_unslash($_POST['location_countrycode'])) : '';
		$location_marker_color	= isset($_POST['location_marker_color']) ? sanitize_text_field(wp_unslash($_POST['location_marker_color'])) : '';
		$location_popuptext     = isset($_POST['location_popuptext'])   ? wp_kses_post(wp_unslash($_POST['location_popuptext'])) : '';
		
        update_post_meta($post_id, 'address_osm',      		$address_osm);
        update_post_meta($post_id, 'location_longitude',	$location_longitude);
        update_post_meta($post_id, 'location_latitude',		$location_latitude);
        update_post_meta($post_id, 'location_address', 		$location_address);
        update_post_meta($post_id, 'location_city', 		$location_city);
        update_post_meta($post_id, 'location_postcode', 	$location_postcode);
        update_post_meta($post_id, 'location_countrycode', 	$location_countrycode);
        update_post_meta($post_id, 'location_popuptext',    $location_popuptext);
        update_post_meta($post_id, 'location_marker_color', $location_marker_color);
		
		
		
//		error_log( print_r( $_POST, true ), 0 );
	}

  	/**
	 * Display the meta box HTML to the user.
	 *
	 * @param WP_Post $post   Post object.
	 */
	public static function html( $post ) {

        wp_nonce_field( basename( __FILE__ ), 'leaflet-location-nonce' );
        
        $lat = esc_attr(get_post_meta( $post->ID, 'location_latitude', true));
        $lng = esc_attr(get_post_meta( $post->ID, 'location_longitude', true));
		
		if($lat == "" || $lng == "") {
			// new or incomplete coordinates - start in Vienna
			$lat = '48.2062771';
			$lng = '16.3650601';
		}
?>
<div class="dev-null-meta-box" data-autosave="false" data-object-type="post">
    <div class="dev-null-field dev-null-text-wrapper">
        <div class="dev-null-label" id="address_osm-label">
            <label for="address_osm"><?php echo esc_html__( 'Address', 'dev-null-openstreetmap'); ?></label>
        </div>
	</div>
    <div class="auto-search-wrapper loupe">
        <input type="text"
            value="<?php echo esc_attr(get_post_meta( $post->ID, 'address_osm', true)); ?>"
            autocomplete="off"
            id="address_osm"
			name="address_osm"
            class="full-width"
            placeholder="enter the address">
        <div class="auto-results-wrapper">
        </div>
    </div>
    <div class="dev-null-field dev-null-map-wrapper">
        <div class="dev-null-label" id="osm_map-label">
            <label for="osm-map"><?php esc_attr_e( 'Map position', 'dev-null-openstreetmap' ); ?></label>
        </div>
        <div id="osm-map" class="dev-null-map" style="width:100%; height: 400px;">
        </div>
    </div>

    <div class="dev-null-field dev-null-text-wrapper">
        <div class="dev-null-label" id="location_popuptext-label">
            <label for="location_latitude">Popup text</label>
        </div>
        <div class="dev-null-input">

            <?php 
                wp_editor( wp_kses_post(get_post_meta( $post->ID, 'location_popuptext', true)), 'location_texteditor',
                    $settings = array(
                        'textarea_name' => 'location_popuptext',
                        'media_buttons' => true,
						'textarea_rows'	=> 10,
                        'tinymce'   =>  [
                            'force_br_newlines' => true,
                        ],
                    ) );
            ?>
        </div>
    </div>
    <div class="dev-null-field dev-null-text-wrapper">
        <div class="dev-null-label" id="location_latitude-label">
            <label for="location_latitude"><?php esc_attr_e( 'Latitude', 'dev-null-openstreetmap' ); ?></label>
        </div>
        <div class="dev-null-input">
            <input value="<?php echo esc_attr(get_post_meta( $post->ID, 'location_latitude', true)); ?>"  type="text" id="location_latitude" class="dev-null-text" name="location_latitude" aria-labelledby="location_latitude-label">
        </div>
    </div>
    <div class="dev-null-field dev-null-text-wrapper">
        <div class="dev-null-label" id="location_longitude-label">
            <label for="location_longitude"><?php esc_attr_e( 'Longitude', 'dev-null-openstreetmap' ); ?></label>
        </div>
        <div class="dev-null-input">
            <input value="<?php echo esc_attr(get_post_meta( $post->ID, 'location_longitude', true)); ?>"  type="text" id="location_longitude" class="dev-null-text" name="location_longitude" aria-labelledby="location_longitude-label">
        </div>
    </div>
    <div class="dev-null-field dev-null-text-wrapper">
        <div class="dev-null-label" id="location_address-label">
            <label for="location_address"><?php esc_attr_e( 'Address', 'dev-null-openstreetmap' ); ?></label>
        </div>
        <div class="dev-null-input">
            <input  value="<?php echo esc_attr(get_post_meta( $post->ID, 'location_address', true)); ?>" type="text" id="location_address" class="dev-null-text" name="location_address" aria-labelledby="location_address-label">
        </div>
    </div>
    <div class="dev-null-field dev-null-text-wrapper">
        <div class="dev-null-label" id="location_city-label">
			<label for="location_city"><?php esc_attr_e( 'City', 'dev-null-openstreetmap' ); ?></label></div>
        <div class="dev-null-input">
            <input  value="<?php echo esc_attr(get_post_meta( $post->ID, 'location_city', true)); ?>" type="text" id="location_city" class="dev-null-text" name="location_city" aria-labelledby="location_city-label">
        </div>
    </div>
    <div class="dev-null-field dev-null-text-wrapper">
        <div class="dev-null-label" id="location_postcode-label">
            <label for="location_postcode"><?php esc_attr_e( 'Postal code', 'dev-null-openstreetmap' ); ?></label>
        </div>
        <div class="dev-null-input">
            <input  value="<?php echo esc_attr(get_post_meta( $post->ID, 'location_postcode', true)); ?>" type="text" id="location_postcode" class="dev-null-text" name="location_postcode" aria-labelledby="location_postcode-label">
        </div>
    </div>
    <div class="dev-null-field dev-null-text-wrapper">
        <div class="dev-null-label" id="location_countrycode-label">
			<label for="location_countrycode"><?php esc_attr_e( 'Country code', 'dev-null-openstreetmap' ); ?></label>
		</div>
        <div class="dev-null-input">
            <input  value="<?php echo esc_attr(get_post_meta( $post->ID, 'location_countrycode', true)); ?>" type="text" id="location_countrycode" class="dev-null-text" name="location_countrycode" aria-labelledby="location_countrycode-label">
        </div>
    </div>
    <div class="dev-null-field dev-null-select-wrapper">
        <div class="dev-null-label" id="location_marker_color-label">
			<label for="location_marker_color"><?php esc_attr_e( 'Marker Color', 'dev-null-openstreetmap' ); ?></label>
		</div>
        <div class="dev-null-input">
            <select id="location_marker_color" class="dev-null-select" name="location_marker_color" aria-labelledby="location_marker_color-label">
				<option value="">Leaflet default Icon</option>
            <?php
                $icons = \DevNullOpenStreetMap\Leaflet_Icon::get_IconNamesList();
                $color = get_post_meta( $post->ID, 'location_marker_color', true);
                
                foreach($icons as $key => $value) {
					if( $color== $key )
						echo "<option value='" . esc_attr($key) . "' selected>" . esc_attr($value) . "</option>";
					else
						echo "<option value='" . esc_attr($key) . "'>" . esc_attr($value) . "</option>";
                }
            ?>
            </select>
        </div>
    </div>
</div>
<?php

//        wp_enqueue_style( 'tomickigrzegorz.autocomplete');
//        wp_enqueue_script( 'tomickigrzegorz.autocomplete');
//        wp_enqueue_script( 'leaflet-search', DEV_NULL_OPENSTREETMAP_URL . 'assets/js/search.js');

	$script_map = "

    const config = {
      minZoom: 6,
      maxZomm: 18,
    };
    // magnification with which the map will start
    const zoom = 12;

    // coordinates
    const lat = {$lat};
    const lng = {$lng};

     const options = {
        zoomDelta:          0.5,
        zoomSnap:           0.5,
        zoomControl:        true,
        minZoom:            1,
        maxZoom:            18,
        dragging:           true,
        doubleClickZoom:    true,
        boxZoom:            true,
        scrollWheelZoom:    true,
        fullscreenControl:  true,
        fullscreenControlOptions: {
            position:           'topright',
            forceSeparateButton: true,
        },
    };

   // calling map
    const map = L.map('osm-map', options);

    // Used to load and display tile layers on the map
    L.tileLayer('https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png', {
      attribution:
        '&copy; <a href=\"https://www.openstreetmap.org/copyright\">OpenStreetMap</a>',
    }).addTo(map);
    map.setView([lat, lng], zoom);

    // ivalidate size because layout shift in backend
    setTimeout(function(){map.invalidateSize()},250);
    
    const marker = L.marker([lat, lng], {
//      don't add title, we use Leaflet Tooltips
//      title: 'Location',
        draggable: true,
    });
    
    marker.bindTooltip('Location');
    marker.addTo(map).bindPopup('Location');

    // turn on dragging markers
    marker.on('click', (e) => {
    //  e.target.dragging.enable();
      console.log(e);
    });

    marker.on('move', (e) => {
        jQuery('#location_latitude').val(e.latlng.lat);
        jQuery('#location_longitude').val(e.latlng.lng);

      console.log(e.latlng);
    });

	";

	wp_add_inline_script('brunob-leaflet.fullscreen', $script_map);

    }
	
	// setup admin columns
	// 
	public static function manage_columns ( $columns ) {

		// return array_merge ( $columns, array ( 
		return array(
			'cb' 								=> '<input type="checkbox">',
			'title' 							=> __ ('Title', 'dev-null-openstreetmap'),
			'location_marker_color'				=> esc_html__( 'Icon', 'dev-null-openstreetmap' ),
			'location_latitude' 				=> esc_html__( 'Latitude', 'dev-null-openstreetmap' ),
			'location_longitude'    			=> esc_html__( 'Longitude', 'dev-null-openstreetmap' ),
			'taxonomy-leaflet_location_group' 	=> esc_html__( 'Locationgroup', 'dev-null-openstreetmap' ),
			'taxonomy-leaflet_location_type'	=> esc_html__( 'Marker Types', 'dev-null-openstreetmap' ),
			'date' 								=> __ ('Date', 'dev-null-openstreetmap')
		);
	}
	
	public static function manage_custom_columns ( $column, $post_id ) {
		switch ( $column ) {
			case 'location_latitude':
				echo esc_attr(get_post_meta( $post_id, 'location_latitude', true));
				break;
			case 'location_longitude':
				echo esc_attr(get_post_meta( $post_id, 'location_longitude', true));
				break;
			case 'location_marker_color':
				echo esc_attr(get_post_meta( $post_id, 'location_marker_color', true));
				break;
		}
	}
    
    static function admin_enqueue_scripts($hook) {

       if(get_post_type() != DEV_NULL_LEAFLET_LOCATION)
            return;
        
   		wp_enqueue_script( 'leaflet');
		wp_enqueue_style( 'leaflet');
        wp_enqueue_script( 'brunob-leaflet.fullscreen');
		wp_enqueue_style( 'brunob-leaflet.fullscreen');
        
//        wp_enqueue_style( 'tomickigrzegorz.autocomplete');
        wp_enqueue_script( 'tomickigrzegorz.autocomplete');

        wp_register_script( 'leaflet-search',
            DEV_NULL_OPENSTREETMAP_URL . 'assets/js/search.js',
            ['tomickigrzegorz.autocomplete'],
            '',
            [
                'strategy' => 'defer',
                'in_footer' => true,
            ]);
            
        wp_enqueue_script( 'leaflet-search');

        wp_enqueue_style( 'leaflet-icon', DEV_NULL_OPENSTREETMAP_URL . 'assets/css/leaflet-markers.css');

    }
}

add_action( 'admin_enqueue_scripts',
    [ '\DevNullOpenStreetMap\Leaflet_Location_Item', 'admin_enqueue_scripts' ],
    10, 1 );

add_filter('dev_null_leaflet_items', 
	[ '\DevNullOpenStreetMap\Leaflet_Location_Item', 'item_list_points'], 10, 1);

add_filter( 'post_row_actions', 
    [ '\DevNullOpenStreetMap\Leaflet_Location_Item', 'post_row_actions' ],
    10, 2 );

add_filter ( 'manage_' . DEV_NULL_LEAFLET_LOCATION . '_posts_columns',
    [ '\DevNullOpenStreetMap\Leaflet_Location_Item', 'manage_columns' ],
    10, 1 );
add_action ( 'manage_' . DEV_NULL_LEAFLET_LOCATION . '_posts_custom_column',
    [ '\DevNullOpenStreetMap\Leaflet_Location_Item', 'manage_custom_columns' ],
    20, 2 );

add_action ( 'add_meta_boxes_' . DEV_NULL_LEAFLET_LOCATION,
    [ '\DevNullOpenStreetMap\Leaflet_Location_Item', 'add_meta_boxes' ],
    10, 2);

add_action ( 'save_post_' . DEV_NULL_LEAFLET_LOCATION,
    [ '\DevNullOpenStreetMap\Leaflet_Location_Item', 'save' ],
    10, 3);

