<?php

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Adds a submenu page under a custom post type parent.
 */
function dev_null_osm_register_option_page() {
    add_submenu_page(
        'edit.php?post_type=leaflet_location',
        __( 'Dev/Null Leaflet Options Page', 'dev-null-openstreetmap' ),
        __( 'Options', 'dev-null-openstreetmap' ),
        // user role
        'manage_options',
        // menu slug
		'dev-null-osm-option-page',
        // page callback to render
		'dev_null_options_page_callback',
    );
}

add_action( 'admin_menu', 'dev_null_osm_register_option_page' );

function dev_null_options_page_callback(){
	?>
		<div class="wrap">
			<h1><?php esc_html(get_admin_page_title()); ?></h1>
			<form method="post" action="options.php">
				<?php
					settings_fields( 'dev_null_options_settings' ); 	// settings group name
					do_settings_sections( 'dev-null-osm-option-page' ); // just a page slug
					submit_button(); 									// "Save Changes" button
				?>
			</form>
		</div>
	<?php
}

add_action( 'admin_init',  'dev_null_settings_fields' );

function dev_null_settings_fields(){

	// I created variables to make the things clearer
	$page_slug = 'dev-null-osm-option-page';
	$option_group = 'dev_null_options_settings';
/*
	// 1. create section
	add_settings_section(
		'dev_null_osm_section_map', // section ID
		esc_attr__( 'Map defaults', 'dev-null-openstreetmap'),  // title (optional)
		'',                                   // callback function to display the section (optional)
		$page_slug
	);
*/

	add_settings_section(
		'dev_null_osm_section_expert', // section ID
		esc_attr__( 'Expert setting', 'dev-null-openstreetmap'),  // title (optional)
		'',                                     // callback function to display the section (optional)
		$page_slug,
        [
            'after_section' => 'Disabling one or more libraries can solve duplicate library enqueue issues. Use with care, as disabling can also break the map rendering',
        ],
	);
	
	// 2. register fields

/*
    register_setting( 'dev_null_options_settings', 'dev_null_osm_map_zoom',
		[ 
			'type' => 'integer',
			'sanitize_callback' => 'dev_null_sanitize_numeric',
		] );
	register_setting( 'dev_null_options_settings', 'dev_null_osm_map_zoom_min',
		[ 
			'default' => '1', 
			'type' => 'integer',
			'sanitize_callback' => 'dev_null_sanitize_numeric',
		] );
	register_setting( 'dev_null_options_settings', 'dev_null_osm_map_zoom_max',
		[ 
			'type' => 'integer',
			'sanitize_callback' => 'dev_null_sanitize_numeric',
		] );
	register_setting( 'dev_null_options_settings', 'dev_null_osm_map_width',
		[ 
			'type' => 'string',
			'sanitize_callback' => 'dev_null_sanitize_textfield',
		] );

	register_setting( 'dev_null_options_settings', 'dev_null_osm_map_height',
		[ 
			'type' => 'string',
			'sanitize_callback' => 'dev_null_sanitize_textfield',
		] );
*/

    register_setting( 'dev_null_options_settings', 'dev_null_osm_disable_js_leaflet',
		[ 
			'type' => 'integer',
			'sanitize_callback' => 'dev_null_sanitize_checkbox',
		] );
    register_setting( 'dev_null_options_settings', 'dev_null_osm_disable_js_fullscreen',
		[ 
			'type' => 'integer',
			'sanitize_callback' => 'dev_null_sanitize_checkbox',
		] );
    register_setting( 'dev_null_options_settings', 'dev_null_osm_disable_js_autocomplete',
		[ 
			'type' => 'integer',
			'sanitize_callback' => 'dev_null_sanitize_checkbox',
		] );
    register_setting( 'dev_null_options_settings', 'dev_null_osm_disable_js_cluster',
		[ 
			'type' => 'integer',
			'sanitize_callback' => 'dev_null_sanitize_checkbox',
		] );
    register_setting( 'dev_null_options_settings', 'dev_null_osm_disable_js_gpx',
		[ 
			'type' => 'integer',
			'sanitize_callback' => 'dev_null_sanitize_checkbox',
		] );
    register_setting( 'dev_null_options_settings', 'dev_null_osm_disable_js_kml',
		[ 
			'type' => 'integer',
			'sanitize_callback' => 'dev_null_sanitize_checkbox',
		] );

	// 3. add fields
/*
	add_settings_field(
		'dev_null_osm_map_zoom',
		esc_attr__( 'Zoom', 'dev-null-openstreetmap' ),
		'dev_null_number', // function to print the field
		$page_slug,
		'dev_null_osm_section_map', // section ID
		[
			'label_for' => 'dev_null_osm_map_zoom',
			'name' 		=> 'dev_null_osm_map_zoom',
			'text' 		=> __('Default zoom with single location marker', 'dev-null-openstreetmap' ),
			'default'	=> 12,
		]
	);

	add_settings_field(
		'dev_null_osm_map_zoom_min',
		esc_attr__( 'Zoom min (1-18)', 'dev-null-openstreetmap' ),
		'dev_null_number', // function to print the field
		$page_slug,
		'dev_null_osm_section_map', // section ID
		[
			'label_for' => 'dev_null_osm_map_zoom_min',
			'name' 		=> 'dev_null_osm_map_zoom_min',
			'default'	=> 1,
		]
	);

	add_settings_field(
		'dev_null_osm_map_zoom_max',
		esc_attr__( 'Zoom max (1-18)', 'dev-null-openstreetmap'),
		'dev_null_number', // function to print the field
		$page_slug,
		'dev_null_osm_section_map', // section ID
		[
			'label_for' => 'dev_null_osm_map_zoom_max',
			'name' 		=> 'dev_null_osm_map_zoom_max',
			'default'	=> 18,
		]
	);

	add_settings_field(
		'dev_null_osm_map_width',
		esc_attr__( 'Map width', 'dev-null-openstreetmap' ),
		'dev_null_string', // function to print the field
		$page_slug,
		'dev_null_osm_section_map', // section ID
		[
			'label_for' => 'dev_null_osm_map_width',
			'name' 		=> 'dev_null_osm_map_width',
			'default'	=> '100%',
		]
	);

	add_settings_field(
		'dev_null_osm_map_height',
		esc_attr__( 'Map height', 'dev-null-openstreetmap' ),
		'dev_null_string', // function to print the field
		$page_slug,
		'dev_null_osm_section_map', // section ID
		[
			'label_for' => 'dev_null_osm_map_height',
			'name' 		=> 'dev_null_osm_map_height',
			'default'	=> '400px',
		]
	);
*/

	add_settings_field(
		'dev_null_osm_disable_js_leaflet',
		esc_attr__( 'LeafletJS library', 'dev-null-openstreetmap'),
		'dev_null_checkbox', // function to print the field
		$page_slug,
		'dev_null_osm_section_expert', // section ID
		[
			'label_for' => 'dev_null_osm_disable_js_leaflet',
			'name' 		=> 'dev_null_osm_disable_js_leaflet',
			'text' 		=> __('Don\'t link, can solve duplicate library enqueue issues', 'dev-null-openstreetmap'),
			'default'	=> 0,
		]
	);

	add_settings_field(
		'dev_null_osm_disable_js_fullscreen',
		esc_attr__( 'Leaflet fullscreen library', 'dev-null-openstreetmap'),
		'dev_null_checkbox', // function to print the field
		$page_slug,
		'dev_null_osm_section_expert', // section ID
		[
			'label_for' => 'dev_null_osm_disable_js_fullscreen',
			'name' 		=> 'dev_null_osm_disable_js_fullscreen',
			'text' 		=> __('Don\'t link, can solve duplicate library enqueue issues', 'dev-null-openstreetmap'),
			'default'	=> 0,
		]
	);

	add_settings_field(
		'dev_null_osm_disable_js_autocomplete',
		esc_attr__( 'Autocomplete library', 'dev-null-openstreetmap'),
		'dev_null_checkbox', // function to print the field
		$page_slug,
		'dev_null_osm_section_expert', // section ID
		[
			'label_for' => 'dev_null_osm_disable_js_autocomplete',
			'name' 		=> 'dev_null_osm_disable_js_autocomplete',
			'text' 		=> __('Don\'t link, can solve duplicate library enqueue issues', 'dev-null-openstreetmap'),
			'default'	=> 0,
		]
	);

	add_settings_field(
		'dev_null_osm_disable_js_cluster',
		esc_attr__( 'Marker cluster library', 'dev-null-openstreetmap'),
		'dev_null_checkbox', // function to print the field
		$page_slug,
		'dev_null_osm_section_expert', // section ID
		[
			'label_for' => 'dev_null_osm_disable_js_cluster',
			'name' 		=> 'dev_null_osm_disable_js_cluster',
			'text' 		=> __('Don\'t link, can solve duplicate library enqueue issues', 'dev-null-openstreetmap'),
			'default'	=> 0,
		]
	);

	add_settings_field(
		'dev_null_osm_disable_js_gpx',
		esc_attr__( 'GPX library', 'dev-null-openstreetmap'),
		'dev_null_checkbox', // function to print the field
		$page_slug,
		'dev_null_osm_section_expert', // section ID
		[
			'label_for' => 'dev_null_osm_disable_js_gpx',
			'name' 		=> 'dev_null_osm_disable_js_gpx',
			'text' 		=> __('Don\'t link, can solve duplicate library enqueue issues', 'dev-null-openstreetmap'),
			'default'	=> 0,
		]
	);

	add_settings_field(
		'dev_null_osm_disable_js_kml',
		esc_attr__( 'KML library', 'dev-null-openstreetmap'),
		'dev_null_checkbox', // function to print the field
		$page_slug,
		'dev_null_osm_section_expert', // section ID
		[
			'label_for' => 'dev_null_osm_disable_js_kml',
			'name' 		=> 'dev_null_osm_disable_js_kml',
			'text' 		=> __('Don\'t link, can solve duplicate library enqueue issues', 'dev-null-openstreetmap'),
			'default'	=> 0,
		]
	);
}

add_action( 'admin_notices', 'dev_null_settings_notice' );

function dev_null_settings_notice() {

	if(
		isset( $_GET[ 'page' ] ) 
		&& 'dev-null-osm-option-page' == $_GET[ 'page' ]
		&& isset( $_GET[ 'settings-updated' ] ) 
		&& true == $_GET[ 'settings-updated' ]
	) {
		?>
			<div class="notice notice-success is-dismissible">
				<p>
					<strong>Slider settings saved.</strong>
				</p>
			</div>
		<?php
	}

}

// custom callback function to print field HTML
function dev_null_string( $args ) {

	if(! isset($args['name']) || $args['name'] == '')
		return;
	
	$name   = $args['name'];
	$value  = get_option( $name );
	if($value == false || $value == '')
		$value = $args['default'];
	
	$text   = isset($args['text']) ? $args['text'] : '';

	?>
	<label>
			<input type="text"
				id="<?php echo esc_attr($name); ?>"
                name="<?php echo esc_attr($name); ?>"
                value="<?php echo esc_attr($value); ?>" />
	</label>
<?php
}

function dev_null_number( $args ){

	if(! isset($args['name']) || $args['name'] == '')
		return;
	
	$name   = $args['name'];
	$value  = get_option( $name );
	if($value == false || $value == '')
		$value = $args['default'];
	
	$text   = isset($args['text']) ? $args['text'] : '';
	?>
	<label>
			<input id="<?php echo esc_attr($name); ?>" type="number"
                    name="<?php echo esc_attr($name); ?>"
                    value="<?php echo intval($value); ?>" />
	</label>
<?php
}

// custom callback function to print checkbox field HTML
function dev_null_checkbox( $args ) {

	if(! isset($args['name']) || $args['name'] == '')
		return;
	
	$name   = $args['name'];
	$value  = get_option( $name );
	if($value == false || $value == '')
		$value = $args['default'];
	
	$text   = isset($args['text']) ? $args['text'] : '';
	?>
	<label>
			<input 
				id="<?php echo esc_attr($name); ?>"
				type="checkbox"
				name="<?php echo esc_attr($name); ?>"
				value="1" <?php checked( $value, '1' ); ?> />
	</label>
	<?php
}

// custom sanitization function for a checkbox field
function dev_null_sanitize_numeric( $value ) {
	return intval($value);
}

// custom sanitization function for a checkbox field
function dev_null_sanitize_textfield( $value ) {
	return sanitize_text_field(wp_unslash($value));
}

// custom sanitization function for a checkbox field
function dev_null_sanitize_checkbox( $value ) {
	return $value != '' ? '1' : '0';
}

